<?php

namespace WBCR\Delivery\Peshkariki;

use WBCR\Delivery\Base\Delivery;
use WDPK\Delivery\Peshkariki\Checkout;
use WDPK\Delivery\Peshkariki\PeshkarikiApi;
use WDPK\Delivery\Peshkariki\Plugin;

class Peshkariki extends Delivery
{
    const SHIPPING_DELIVERY_ID = 'wbcr_peshkariki_delivery';

    const CHECKOUT_HANDLER = Checkout::class;
    /**
     * @var \WDPK\Delivery\Peshkariki\PeshkarikiApi
     */
	private $api;

	public function __construct( $instance_id = 0 ) {

		$this->api = new PeshkarikiApi();
		parent::__construct( $instance_id );
	}

	/**
	 * {@inheritDoc}
	 */
	public static function admin_enqueue_scripts() {
		wp_enqueue_script(
			self::SHIPPING_DELIVERY_ID . '-admin',
			WDPK_PLUGIN_URL . '/admin/assets/js/admin.js',
			[ 'jquery' ],
			static::app()->getPluginVersion()
		);

        wp_enqueue_style(
	        self::SHIPPING_DELIVERY_ID . '-admin-css',
	        WDPK_PLUGIN_URL . '/admin/assets/css/admin.css',
	        [],
	        static::app()->getPluginVersion()
        );

        wp_localize_script(
            self::SHIPPING_DELIVERY_ID . '-admin',
            'localStrings',
            [
	            'inner'   => __( 'Inner: ', 'wd-peshkariki-delivery' ),
	            'outer'   => __( 'Outer: ', 'wd-peshkariki-delivery' ),
	            'package' => __( 'Package: ', 'wd-peshkariki-delivery' ),
	            'max'     => __( 'Max: ', 'wd-peshkariki-delivery' ),
            ]
        );
    }

    public function init() {
	    $this->title              = __( 'Peshkariki', 'wd-peshkariki-delivery' );
	    $this->method_title       = __( 'Peshkariki', 'wd-peshkariki-delivery' );
	    $this->method_description = __( 'Peshkariki extension adds shipping method to Checkout page.', 'wd-peshkariki-delivery' );


	    parent::init(); // TODO: Change the autogenerated stub
    }

    public static function app() {
	    return wdpk_get_current_plugin();
    }


    public function calculate_shipping( $package = [] ) {
        $price = $this->api->calculate_shipping($package);

        $this->add_rate( [
            'label'   => $this->title,
            'cost'    => isset( $price ) ? (float) $price : 1,
            'taxes'   => false,
            'package' => $package,
        ] );
    }

    public function settings_form_fields()
    {
        return array_merge(
            [
                'client_login' => [
	                'title'             => __( 'Login', 'wd-peshkariki-delivery' ),
	                'type'              => 'text',
	                'default'           => '',
	                'desc_tip'          => false,
	                'description'       => __( 'Customer\'s login in Peshkariki service', 'wd-peshkariki-delivery' ),
	                'custom_attributes' => [ 'autocomplete' => 'off', 'required' => 'required' ]
                ],
                'client_password' => [
	                'title'             => __( 'Password', 'wd-peshkariki-delivery' ),
	                'type'              => 'text',
	                'default'           => '',
	                'desc_tip'          => false,
	                'description'       => __( 'Customer\'s password in Peshkariki service', 'wd-peshkariki-delivery' ),
	                'custom_attributes' => [ 'autocomplete' => 'off', 'required' => 'required' ]
                ],
                'client_name' => [
	                'title'             => __( 'Name', 'wd-peshkariki-delivery' ),
	                'type'              => 'text',
	                'default'           => '',
	                'desc_tip'          => false,
	                'description'       => '',
	                'custom_attributes' => [ 'autocomplete' => 'off', 'required' => 'required' ]
                ],
                'client_phone' => [
	                'title'             => __( 'Phone', 'wd-peshkariki-delivery' ),
	                'type'              => 'text',
	                'default'           => '',
	                'desc_tip'          => false,
	                'description'       => '',
	                'custom_attributes' => [ 'autocomplete' => 'off', 'required' => 'required' ]
                ],
                'shop_city_id' => [
	                'title'             => __( 'City', 'wd-peshkariki-delivery' ),
	                'type'              => 'select',
	                'default'           => '1',
	                'desc_tip'          => false,
	                'options'           => PeshkarikiApi::CITIES,
	                'description'       => __( 'City where the store is located', 'wd-peshkariki-delivery' ),
	                'custom_attributes' => [ 'autocomplete' => 'off', 'required' => 'required' ]
                ],
                'shop_street' => [
	                'title'             => __( 'Street', 'wd-peshkariki-delivery' ),
	                'type'              => 'text',
	                'default'           => '',
	                'desc_tip'          => false,
	                'description'       => __( 'Street where the store is located', 'wd-peshkariki-delivery' ),
	                'custom_attributes' => [ 'autocomplete' => 'off', 'required' => 'required' ]
                ],
                'shop_building' => [
	                'title'             => __( 'Building', 'wd-peshkariki-delivery' ),
	                'type'              => 'text',
	                'default'           => '',
	                'desc_tip'          => false,
	                'description'       => __( 'Building where the store is located' ),
	                'custom_attributes' => [ 'autocomplete' => 'off', 'required' => 'required' ]
                ],
                'devmode_enabled' => [
	                'title'             => __( 'Developer Mode', 'wd-peshkariki-delivery' ),
	                'type'              => 'checkbox',
	                'default'           => '',
	                'desc_tip'          => false,
	                'description'       => __( 'Is developer mode enabled' ),
	                'custom_attributes' => [ 'autocomplete' => 'off' ]
                ],

            ],
	        parent::settings_form_fields()
        );
    }

	/**
	 * @param $order \WC_Order
	 */
	public static function order_data_output( $order ) {
		$order_meta      = $order->get_meta( Order::DELIVERY_CHECKOUT_META_KEY );
		$shipping_method = explode( ':', $order_meta['shipping_method'][0] );
		$shipping_method = $shipping_method[0];

		if ( $shipping_method == self::SHIPPING_DELIVERY_ID ) {
			$price   = $order->get_shipping_total();
			$country = isset( $order_meta['billing_country'] ) ? $order_meta['billing_country'] : '';
			$city    = isset( $order_meta['billing_city'] ) ? $order_meta['billing_city'] : '';
			$state   = isset( $order_meta['billing_state'] ) ? $order_meta['billing_state'] : '';
			$addr    = isset( $order_meta['billing_address_1'] ) ? $order_meta['billing_address_1'] : '';
			$room    = isset( $order_meta['billing_address_2'] ) ? $order_meta['billing_address_2'] : '';
			$address = "{$state}, {$city}, {$addr}, {$room}";

			?>
            </div></div>
            <div style='display: inline-block; width: 100%;'>
            <h3>Выбранный способ доставки</h3>
            <table>
                <tr class="woocommerce-cdek-delivery1">
                    <td colspan="2">
                        <div id='cdek-delivery-selected' class=''>
                            <div id='cdek-delivery-selected-content'>
                                <div class='delivery-selected'>
                                    <div class='delivery-flex-row'>
                                        <div class='delivery-flex-col'>
                                            <div class='delivery-selected-content'>
                                                <div class='delivery-selected-title'>Служба доставки</div>
                                                <div>
                                                    <div class='delivery-selected-image service-image'></div>
                                                    <div class='delivery-selected-name'> <?= __( 'Пешкарики', 'wd-peshkariki-delivery' ); ?> </div>
                                                </div>
                                            </div>
                                        </div>
                                        <div class='delivery-flex-col'>
                                            <div class='delivery-selected-content'>
                                                <div class='delivery-selected-title'>Адрес доставки</div>
                                                <div class='delivery-selected-value'><?= $address; ?></div>
                                            </div>
                                        </div>
                                        <div class='delivery-flex-col'>
                                            <div class='delivery-selected-content'>
                                                <div class='delivery-selected-title'>Стоимость доставки</div>
                                                <div class='delivery-selected-value'><?php echo $price . " " . get_woocommerce_currency_symbol(); ?>
                                                </div>
                                            </div>
                                        </div>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </td>
                </tr>
            </table>
			<?php
		}
	}


}
